import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.util.Date;
import java.util.Hashtable;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * RadioClock by jibHaine
 * http://www.jibhaine.fr
 *
 */
@SuppressWarnings("serial")
public class RadioClock extends JFrame{
	
	//PI pour les calculs d'angle
	private double PI = 3.1415926535897932384626433832795 ;
	//angle entre chaque seconde
	private double angle = (6*PI)/180;
	//variable qui change pour faire le tour initialisé a pi/2 
	private double O = -(PI/2);

	//position du centre:
	private int circleX=(int) (getToolkit().getScreenSize().width)/2;
	private int circleY=(int) (getToolkit().getScreenSize().height)/2 ;
	//taille du cercle:
	private int rayon=(int) (getToolkit().getScreenSize().height)/2 - 30;
	//position des chiffres de l'heure
	private int timeX = circleX-250;
	private int timeY = circleY-70;

	//Collection d'images
	private Hashtable<String,Image> img = new Hashtable<String,Image>();
	//temps entre deux rafraichissements (en ms)
	private int refreshTime=300;
	
	//booleens affichage
	boolean 
		showHelp = false, 
		showNumbers=true, 
		showLeds=true,
		showDarkLeds=true;
	
    /**
     * Initialisation de la fenetre en plein écran
     */
	public RadioClock(){
		super("RadioClock");
		//initialisation des Images
		for(int i=0;i<10;i++) img.put(Integer.toString(i),new ImageIcon("images/"+i+".png").getImage());
		img.put(":",new ImageIcon("images/points.png").getImage());
		img.put("on",new ImageIcon("images/led_on.png").getImage());
		img.put("off",new ImageIcon("images/led_off.png").getImage());
		
		
		//initialisation de la fenêtre
		setSize((int)(getToolkit().getScreenSize().width),(int)(getToolkit().getScreenSize().height));
		setLocation(0,0);
		setBackground(new Color(Color.TRANSLUCENT));
		setUndecorated(true);
		setResizable(false);
		setAlwaysOnTop(true);
		
		//masquage du cursor
		Toolkit tk = Toolkit.getDefaultToolkit();
		Cursor cur = tk.createCustomCursor(new ImageIcon("").getImage(), new Point(1, 1), "mon oeil");
		setCursor(cur);
		
		//écouteurs de clavier
		addKeyListener(new RadioClockListener());
		addMouseListener(new RadioClockListener());
		addMouseWheelListener(new RadioClockListener());
		
		//ajout de l'ecran
		add(new RadioClockScreen());
		setVisible(true);
	}
	/**
	 * l'écran sur lequel est dessiné l'horloge
	 * 
	 * @author jibHaine
	 *
	 */
	public class RadioClockScreen extends JPanel {

		public RadioClockScreen(){
			setBackground(Color.BLACK);
			//thread de rafraichissement toutes les demi-secondes pour éviter le lag 
			new Thread(){
				public void run(){
					
					while(true){
						repaint();
						try {
							sleep(refreshTime);
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
				}
			}.start();
		}
		/**
		 * Affiche les images à l'écran
		 */
		public void paintComponent(Graphics g) {
			
			O = -(PI/2);
			int x=0, y=0;
			//date du jour
			Date d = new Date();
			int sec=d.getSeconds();
			int hour=d.getHours();
			int min=d.getMinutes();

			String h = Integer.toString(hour);
			String m = Integer.toString(min);

			setForeground(Color.white);

			super.paintComponent(g);
			// On convertit le contexte graphique en Graphics2D
			Graphics2D g2 = (Graphics2D) g;

			if(showDarkLeds){
				//affichage de led noire toutes les 5 secondes
				for(int i=0;i<12;i++){
					//calcul des coordonnées
					x=circleX+(int)(rayon*Math.cos(O));
					y=circleY+(int)(rayon*Math.sin(O)) - 30;
	
					g2.drawImage(img.get("off"), x, y, this);

					O=O+(30*PI)/180;
				}
			}
			
			O = -(PI/2);
			
			
			if(showLeds){
				//affichage des secondes
				for(int i=0;i<60;i++){
					x=circleX+(int)(rayon*Math.cos(O));
					y=circleY+(int)(rayon*Math.sin(O)) - 30;
					//si la seconde parcourue est passée on l'affiche en rouge
					if(i<=sec){
						g2.drawImage(img.get("on"), x, y, this);
					}
					
					O=O+angle;
				}
			}
			if (showNumbers){
				//affichage des chiffres
				//les heures
				if(h.length()<2){//s'il n'ya qu'un chiffre on affiche un 0 devant
					g2.drawImage(img.get("0"),timeX+0*100,timeY, this);
					g2.drawImage(img.get(Character.toString(h.charAt(0))),timeX+1*100,timeY, this);
	
				}else{
					g2.drawImage(img.get(Character.toString(h.charAt(0))),timeX+0*100,timeY, this);
					g2.drawImage(img.get(Character.toString(h.charAt(1))),timeX+1*100,timeY, this);
				}
	
				//séparateur :
				g2.drawImage(img.get(":"),timeX+2*100+25,timeY, this);
				
				//les minutes
				if(m.length()<2){//s'il n'ya qu'un chiffre on affiche un 0 devant
					g2.drawImage(img.get("0"),timeX+3*100,timeY, this);
					g2.drawImage(img.get(Character.toString(m.charAt(0))),timeX+4*100,timeY, this);
	
				}else{
					g2.drawImage(img.get(Character.toString(m.charAt(0))),timeX+3*100,timeY, this);
					g2.drawImage(img.get(Character.toString(m.charAt(1))),timeX+4*100,timeY, this);
				}

			}
			
			if (showHelp){
				int dd = 18;
				g2.drawString("---------- RadioClock ------",10,1*dd);
				g2.drawString(" développé à l'arrache      ",10,2*dd);
				g2.drawString(" par jibHaine               ",10,3*dd);
				g2.drawString(" (www.jibHaine.fr)          ",10,4*dd);
				g2.drawString("                            ",10,5*dd);
				g2.drawString("---------- Commandes -------",10,6*dd);
				g2.drawString("                            ",10,7*dd);
				g2.drawString(" a:affiche cette aide       ",10,8*dd);
				g2.drawString("                            ",10,9*dd);
				g2.drawString(" l:affiche/masque les leds  ",10,10*dd);
				g2.drawString(" h:affiche/masque l'heure   ",10,11*dd);
				g2.drawString(" o:affiche/masque l'ombre   ",10,12*dd);
				g2.drawString("                            ",10,13*dd);
				g2.drawString(" q:quitter radioclock       ",10,14*dd);
				g2.drawString("                            ",10,15*dd);
				g2.drawString("----------------------------",10,16*dd);
			}
		}
	}
	/**
	 * Ecouteurs de souris et de clavier pour quitter / interargir avec l'horloge
	 * @author JBN
	 *
	 */
	public class RadioClockListener implements MouseWheelListener,MouseListener,KeyListener{
		
		public RadioClockListener(){}
		
		@Override
		public void mouseWheelMoved(MouseWheelEvent e){
			System.out.println(e.getClickCount());
			rayon+=e.getWheelRotation()*10;
		}
		
		public void mouseClicked(MouseEvent e){System.exit(0);}
		public void mouseEntered(MouseEvent e){}
		public void mouseExited(MouseEvent e){}
		public void mousePressed(MouseEvent e){}
		public void mouseReleased(MouseEvent e){System.exit(0);}
		
		
		public void keyPressed(KeyEvent e){}
		public void keyReleased(KeyEvent e){}
		public void keyTyped(KeyEvent e){
			char c = e.getKeyChar();
			switch(c){
				case 'a' : showHelp = !showHelp; break;
				case 'h' : showNumbers = !showNumbers; break;
				case 'l' : showLeds = !showLeds; break;
				case 'o' : showDarkLeds = !showDarkLeds; break;
				case 'q' : System.exit(0); break;
			}
			
		}

	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		new RadioClock();
	}
}


